﻿; textmode client to show all relevant linux system informations

EnableExplicit

; the basic textmode part of this idea: https://www.purebasic.fr/english/viewtopic.php?f=15&t=71646&sid=b83071c81bbfbda6034bb063b6a7c898

;  tested with
; ubuntu 18.10
; CentOS
; Sabayon 18.05
; Debian 9 (stretch)

; find my public IP adress, the one that other computers see when i connect to them. This often depends on NAT

Define Settings.i, NetworkResult.i, Result.I
Define Parameters.i, Position1.i
Define IPV4.q, IPV6.q
Define LoopCounter.i
Define Message$, Parameter$, MyName$, FieldValue$, Output$
Global AppFlags.i
Global Dim SystemInfo.s(20)
Global IPV4$, IPV6$, Param1$
Global GTK2Version$, GTK3Version$

XIncludeFile "SystemInfo.pbi"

; returns strings for the V4 and V6 IP adress of this machine

Procedure Add2Log(Message$)
  Define FileID.i, Result.i
  
  FileID = OpenFile(#PB_Any, #SystemInfo_LogfileName, #PB_File_Append | #PB_Unicode)
  
  If (FileID <> 0)
    Result = WriteStringN(FileID, Message$, #PB_Unicode)
    CloseFile(FileID)
    If (Result = 0)
      Debug "couldn't append '" + Message$ + "' to logfile"
    EndIf
  EndIf
  
EndProcedure

; return value is the amount of IP types i could find or negative if something failed if no IP at all can be found (for example - no network connection exists)

; 1 - got IPV4
;
; -1 - couldn't get IP
; -3 - failed to get memory to read the temporary file
; -4 - failed to read the temporary html file
; -5 - couldn't locate the correct html tag

Procedure.i GetMyIP(*IPV4, *IPV6)
  
  Define Result.i, FileID.i, FSize.i, ReturnValue.i
  Define Position1.i, Position2.i
  Define *Buffer
  Define MSG$, TempFileName$, SearchToken$
  Define HtmlFile$, IPAddress$
  
  ; preset the result values
  
  IPV4$ = ""
  IPV6$ = ""
  
  TempFileName$ = GetTemporaryDirectory() + "systeminfo.htm"
  
  DeleteFile(TempFileName$)
  
  Result = ReceiveHTTPFile(#SystemInfo_GetIPService, TempFileName$)
  
  If (Result = 0)
    ProcedureReturn -1
  EndIf
  
  ReturnValue = 1
  
  ; now find my IP in the file
  
  FileID = ReadFile(#PB_Any, TempFileName$)
  
  If (FileID <> 0)
    FSize = FileSize(TempFileName$)
    
    If (FSize > 0)
      *Buffer = AllocateMemory(FSize)
      If (*Buffer = 0)
        CloseFile(FileID)
        ReturnValue = -3
      EndIf
      
      ReturnValue = 0
      
      Result = ReadData(FileID, *Buffer, FSize)
      If (Result = FSize)
        SearchToken$ = "IP: "
        HtmlFile$ = PeekS(*Buffer, FSize, #PB_Ascii)
        Position1 = FindString(HtmlFile$, SearchToken$)
        If (Position1 <> 0)
          Position1 = FindString(HtmlFile$, ">", Position1)
          If (Position1 <> 0)
            Position1 + StringByteLength(">")
          EndIf
        EndIf
          
        Position2 = FindString(HtmlFile$, "</span>", Position1)
        
        If ((Position1 <> 0) And (Position2 <> 0))
          IPV4$ = Mid(HtmlFile$, Position1, Position2 - Position1)
          ReturnValue = 1
        Else
          ReturnValue = -5
        EndIf
        
      Else
        ReturnValue = -4
      EndIf
      
      FreeMemory(*Buffer)
      *Buffer = 0
    EndIf
    
    CloseFile(FileID)
    FileID = 0
  Else
    ReturnValue = -2
  EndIf
  
  ; cleanup
  DeleteFile(TempFileName$)
  
  ProcedureReturn ReturnValue
  
EndProcedure

Procedure SystemInfo_Init()
  AppFlags = #SYSTEMINFO_Verbose
  IPV4$ = ""
  IPV6$ = ""
EndProcedure

Procedure SystemInfo_Print(Message$)
  If ((AppFlags & #SYSTEMINFO_ConsoleInited) = 0)
    If (OpenConsole() = 0)
      End 999
    EndIf
    
    AppFlags = AppFlags | #SYSTEMINFO_ConsoleInited
  EndIf
  
  PrintN(Message$)
EndProcedure

Procedure GetOSRelease()
  
  Define FileID.i, ComposedCodeName.i
  Define ThisLine$, Field$, FieldValue$
  
  ComposedCodeName = 0
  
  FileID.i = ReadFile(#PB_Any, "/etc/os-release")
  
  If (FileID = 0)
    ProcedureReturn -1
  EndIf
  
  Add2Log("Content of os-release")
  While (Eof(FileID) = 0)
    ThisLine$ = ReadString(FileID)
    Add2Log(ThisLine$)
    Field$ = LCase(StringField(ThisLine$, 1, "="))
    FieldValue$ = ReplaceString(StringField(ThisLine$, 2, "="), Chr(34), "")
    Select Field$
      Case ""
        
      Case "name"
        SystemInfo(#SystemInfoField_Name) = FieldValue$
        Select FieldValue$
          Case "Ubuntu", "Debian GNU/Linux", "Linux Mint"
            SystemInfo(#SystemInfoField_PackageManager) = "apt-get"
            
          Case "CentOS Linux"
            SystemInfo(#SystemInfoField_PackageManager) = "yum"
            
          Case "Sabayon"
            SystemInfo(#SystemInfoField_PackageManager) = "equo"
            
          Case "Mageia"
            SystemInfo(#SystemInfoField_PackageManager) = "urpmi"
            
          Default
            PrintN("unhandled name is '" + FieldValue$ + "'")
        EndSelect
        
      Case "version"
        SystemInfo(#SystemInfoField_Version) = FieldValue$
        
      Case "id"
        SystemInfo(#SystemInfoField_ID) = FieldValue$
        
      Case "id_like"
        SystemInfo(#SystemInfoField_IDLike) = FieldValue$
        
      Case "pretty_name"
        SystemInfo(#SystemInfoField_PrettyName) = FieldValue$
        
      Case "home_url"
        SystemInfo(#SystemInfoField_URL) = FieldValue$
        
      Case "version_id"
        SystemInfo(#SystemInfoField_VersionID) = FieldValue$
        
      Case "bug_report", "privacy_url", "home_url", "privacy_policy_url", "bug_report_url", "support_url"
        
      Case "documentation_url"
        
      Case "redhat_bugzilla_product"
        If (SystemInfo(#SystemInfoField_CodeName)) = ""
          SystemInfo(#SystemInfoField_CodeName) = FieldValue$
          ComposedCodeName = 1
        EndIf
        
      Case "redhat_bugzilla_product_version"
        If (ComposedCodeName = 1)
          SystemInfo(#SystemInfoField_CodeName) + " " + FieldValue$
          ComposedCodeName = 2
        EndIf
      Case "version_codename"
        SystemInfo(#SystemInfoField_CodeName) = FieldValue$
        
      Case "ubuntu_codename"
        SystemInfo(#SystemInfoField_UbuntuCodeName) = FieldValue$
        
      Case "platform_id"
        SystemInfo(#SystemInfoField_PlatformID) = FieldValue$
        
      ; unhandled fields - just to suppress warnings about them
      Case "ansi_color", "centos_mantisbt_project", "centos_mantisbt_project_version"
      Case "redhat_support_product", "redhat_support_product_version"
        
      Case "cpe_name"
        SystemInfo(#SystemInfoField_CodeName) = FieldValue$
        
      Case "variant"
        SystemInfo(#SystemInfoField_VariantID) = FieldValue$
        
      Case "variant_id"
        
      Default
        If ((AppFlags & #SystemInfo_Verbose) = #SystemInfo_Verbose)
          PrintN("unhandled field '" + Field$ + "' with value '" + FieldValue$ + "'")
;          Debug "unhandled field '" + Field$ + "' with value '" + FieldValue$ + "'"
        EndIf
    EndSelect
        
;    Debug "got " + Field$+ " with '" + FieldValue$ + "'"
  Wend
  
  Add2Log("Content of os-release ---------- end")
  CloseFile(FileID)
  
  ProcedureReturn 1
  
EndProcedure

Procedure.s GetContent(FileName$, *ReturnCode = 0)
  Define FileID.i
  Define Content$
  
  Content$ = ""
  If (*ReturnCode <> 0)
    PokeB(*ReturnCode, 0)
  EndIf
  
  FileID = ReadFile(#PB_Any, FileName$, #PB_Ascii)
  If (FileID <> 0)
    If (*ReturnCode <> 0)
      PokeB(*ReturnCode, 1)
    EndIf
    
    While (Eof(FileID) = 0)
      If (Content$ <> "")
        Content$ + Chr(13)
      EndIf
      
      Content$ + ReadString(FileID)
    Wend
    
    CloseFile(FileID)
    FileID = 0
  EndIf
  
  ProcedureReturn Content$
EndProcedure

Procedure.s GetOutput(ProgramName$, Parameter$, *ExitCode, SkipLines.i = 0)
  Define ThisTool.i, ExitCode.i, LineNum.i
  Define Output$, NextLine$
  
  ThisTool = RunProgram(ProgramName$, Parameter$, GetCurrentDirectory(), #PB_Program_Open | #PB_Program_Read | #PB_Program_Ascii)
  
  Output$ = ""
  LineNum = 1
  If (ThisTool <> 0)
    While ProgramRunning(ThisTool)
      If AvailableProgramOutput(ThisTool)
        NextLine$ = ReadProgramString(ThisTool)
        If (SkipLines < LineNum)
          If (Len(Output$) > 0)
            Output$ + Chr(13)
          EndIf
          
          Output$ + NextLine$
        EndIf
        
        Linenum + 1
      EndIf
      
    Wend
    ExitCode = ProgramExitCode(ThisTool)
    CloseProgram(ThisTool)
    
    ExitCode = 1
    
    If (*ExitCode <> 0)
      PokeI(*ExitCode, ExitCode)
    EndIf
  Else
    If (*ExitCode <> 0)
      PokeI(*ExitCode, ThisTool)
    EndIf
  EndIf
  
  ProcedureReturn Output$
  
EndProcedure

Procedure.s Equo_GetVersion(SearchString$)
  Define f.i
  Define Position1.i,Position2.i
  Define c$
  Define CommandName$, Output$, Version$
  
  CommandName$ = GetOutput("which", "equo", 0)
  
  Output$ = GetOutput(CommandName$, "search " + SearchString$, 0)
  
  Position1 = FindString(Output$, "Installed: ")
  If (Position1 = 0)
    ProcedureReturn ""
  EndIf
  
  Position1 = FindString(Output$, "version:", Position1, #PB_String_NoCase)
  
  If (Position1 <> 0)
    Position1 + 8
    Position2 = FindString(Output$, Chr(13), Position1)
    If (Position2 <> 0)
      Version$ = LTrim(Mid(Output$, Position1, Position2 - Position1 - 1))
      For f = 1 To Len(Version$)
        c$ = Mid(Version$, f, 1)
        If (c$ = " ")
          Version$ = Left(Version$, f - 1)
          Break
        EndIf
      Next
      
      ProcedureReturn Version$
    EndIf
  EndIf
  
  ProcedureReturn ""
  
EndProcedure

Procedure.s GrabVersionNumber(Output$)
  Define Position1.i, Position2.i
  Define Version$
  
  Version$ = "unknown"
  Position1 = FindString(Output$, "Installed Packages")
  If (Position1 = 0)
    ; we try an emergency fallback to german, due to me being german and some automatism accidentially chose that as install language for fedora 29 :s
    Position1 = FindString(Output$, "Installierte Pakete")
    Position2 = FindString(Output$, "Verfügbare Pakete")
  Else
    Position2 = FindString(Output$, "Available Packages")
  EndIf
  
  If ((Position1 > 0) And (Position2 > 0))
    Position1 = FindString(Output$, "Version", Position1, #PB_String_NoCase)
    If (Position1 > 0)
      Position1 = FindString(Output$, ": ", Position1)
      Position2 = FindString(Output$, Chr(13), Position1)
      If ((Position1 > 0) And (Position2 > 0))
        Version$ = Mid(Output$, Position1 + 2, Position2 - Position1 - 2)
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn Version$
  
EndProcedure

Procedure GetGTKVersion_URPMI()
; mageia
; /usr/bin/urpmq
; [root@localhost gtk-2.0]# urpmq --summary gtk+3.0
; gtk+3.0 : The GIMP ToolKit (GTK+), a library For creating GUIs ( 3.22.16-1.mga6 )
; gtk+3.0 : The GIMP ToolKit (GTK+), a library For creating GUIs ( 3.22.16-1.mga6 )
; [root@localhost gtk-2.0]# urpmq --summary gtk+2.0
; gtk+2.0 : The GIMP ToolKit (GTK+), a library For creating GUIs ( 2.24.31-3.mga6 )
; gtk+2.0 : The GIMP ToolKit (GTK+), a library For creating GUIs ( 2.24.31-3.mga6 )
  
  Define Result.i, FoundFields.i
  Define i1.i, i2.i, i3.i, i4.i
  Define c$
  Define OneLine$, Name$, Version$, Architecture$, Description$
  
  If (SystemInfo(#SystemInfoField_PackageManager) <> "urpmi")
    PrintN("Capital program flow bug in textmode")
    ProcedureReturn
  EndIf
  
  Result = RunProgram("/usr/bin/urpmq", "--summary gtk+2.0", GetCurrentDirectory(), #PB_Program_Open | #PB_Program_Read | #PB_Program_Ascii)
  OneLine$ = ""
  If (Result <> 0)
    FoundFields = -1
  EndIf
  
  While ProgramRunning(Result)
    If AvailableProgramOutput(Result)
      OneLine$ = ReadProgramString(Result)
      
      If (FoundFields = -1)
        i1 = FindString(OneLine$, ".mga6")
        If (i1 <> 0)
          ; backwards scan till we find a space
          i2 = i1
          While (i2 > 1)
            i2 - 1
            
            c$ = Mid(OneLine$, i2, 1)
            If (c$ = " ")
              SystemInfo(#SystemInfoField_GTK2Version) = Mid(OneLine$, i2 + 1, i1 - i2 - 1)
              Break
            EndIf
          Wend
          
          
        EndIf
      EndIf
        
    EndIf
  Wend
  ;    Debug "Exitcode: " + Str(ProgramExitCode(Result))
  
  Result = RunProgram("/usr/bin/urpmq", "--summary gtk+3.0", GetCurrentDirectory(), #PB_Program_Open | #PB_Program_Read | #PB_Program_Ascii)
  OneLine$ = ""
  If (Result <> 0)
    FoundFields = -1
  EndIf
  
  While ProgramRunning(Result)
    If AvailableProgramOutput(Result)
      OneLine$ = ReadProgramString(Result)
      
      If (FoundFields = -1)
        i1 = FindString(OneLine$, ".mga6")
        If (i1 <> 0)
          ; backwards scan till we find a space
          i2 = i1
          While (i2 > 1)
            i2 - 1
            
            c$ = Mid(OneLine$, i2, 1)
            If (c$ = " ")
              SystemInfo(#SystemInfoField_GTK3Version) = Mid(OneLine$, i2 + 1, i1 - i2 - 1)
              Break
            EndIf
          Wend
          
          
        EndIf
      EndIf
        
    EndIf
  Wend
    
  CloseProgram(Result) ; Close the connection to the program
EndProcedure

Procedure GetGTKVersion_AptGet()
  Define Result.i, FoundFields.i
  Define i1.i, i2.i, i3.i, i4.i
  Define OneLine$, Name$, Version$, Architecture$, Description$
  
  If (SystemInfo(#SystemInfoField_PackageManager) <> "apt-get")
    PrintN("Capital program flow bug in textmode")
    ProcedureReturn
  EndIf
  
  Result = RunProgram("/usr/bin/dpkg", "-l libgtk*", GetCurrentDirectory(), #PB_Program_Open | #PB_Program_Read | #PB_Program_Ascii)
  OneLine$ = ""
  If (Result <> 0)
    FoundFields = -1
    While ProgramRunning(Result)
      If AvailableProgramOutput(Result)
        OneLine$ = ReadProgramString(Result)
        
        If (FoundFields = -1)
          i1 = FindString(OneLine$, "Name")
          If (i1 <> 0)
            i2 = FindString(OneLine$, "Version")
            i3 = FindString(OneLine$, "Architecture")
            i4 = FindString(OneLine$, "Description")
            If ((i2 <> 0) And (i3 <> 0) And (i4 <> 0))
              FoundFields = 1
            EndIf
          EndIf
        Else
          Name$ = RTrim(Mid(OneLine$, i1, (i2 - i1)))
          Version$ = RTrim(Mid(OneLine$, i2, (i3 - i2)))
          Architecture$ = RTrim(Mid(OneLine$, i3, (i4 - i3)))
          Description$ = RTrim(Mid(OneLine$, i4))
          If (FindString(Description$, "GTK+ graphical user interface library") = 1)
            If (Len(Version$) > 1)
              If (Left(Version$, 2) = "2.")
                SystemInfo(#SystemInfoField_GTK2Version) = Version$
              EndIf
              
              If (Left(Version$, 2) = "3.")
                SystemInfo(#SystemInfoField_GTK3Version) = Version$
              EndIf
            EndIf
          EndIf
        EndIf
        
      EndIf
    Wend
    ;    Debug "Exitcode: " + Str(ProgramExitCode(Result))

    CloseProgram(Result) ; Close the connection to the program
  EndIf
EndProcedure

Procedure GetQTVersion()
  Define CommandName$, Output$, FirstLine$
  
  ; see https://stackoverflow.com/questions/23058718/how-to-find-version-of-qt  
  
  CommandName$ = GetOutput("which", "qtdiag", 0)
  If (CommandName$ <> "")
    Output$ = GetOutput(CommandName$, "", 0)
    
    FirstLine$ = StringField(Output$, 1, Chr(10))
    If (FirstLine$ <> "")
;      SystemInfo_Print("Got '" + FirstLine$ + "'")
      SystemInfo(#SystemInfoField_QTVERSION) = StringField(FirstLine$, 2, " ")
    EndIf
  EndIf
  
EndProcedure


;-- Main start

DeleteFile(#SystemInfo_LogfileName)

SystemInfo_Init()

; SystemInfo_Print(Message$)

; If (Result > 0)
;   For LoopCounter = 1 To Result
;     Parameter$ = ProgramParameter(LoopCounter - 1)
;     Message$ = "Parameter " + Str(LoopCounter) + " is '" + Parameter$ + "'"
;     SystemInfo_Print(Message$)
;   Next
; EndIf

GetOSRelease()

NetworkResult = InitNetwork()

If (NetworkResult <> 0)
  Result = GetMyIP(@IPv4, @IPV6)
  
  Select Result
    Case 0
      ; we just don't output any info in this case
      
    Case 1
      
    Default
      
  EndSelect

EndIf

; If (SystemInfo(#SystemInfoField_ID) = "debian")
; EndIf

If (SystemInfo(#SystemInfoField_ID) = "sabayon")
  
  If (SystemInfo(#SystemInfoField_IDLike) = "")
    SystemInfo(#SystemInfoField_IDLike) = "Gentoo"
  EndIf
  
  If (SystemInfo(#SystemInfoField_Version) = "")
    SystemInfo(#SystemInfoField_Version)  = GetContent("/etc/sabayon-edition")
  EndIf
  
  If (SystemInfo(#SystemInfoField_VersionID) = "")
    SystemInfo(#SystemInfoField_VersionID)  = GetContent("/etc/sabayon-release")
  EndIf
  
  If (SystemInfo(#SystemInfoField_GTK2Version) = "")
    Systeminfo(#SystemInfoField_GTK2Version) = Equo_Getversion("x11-libs/gtk+-2")
  EndIf
  
  If (SystemInfo(#SystemInfoField_GTK3Version) = "")
    Systeminfo(#SystemInfoField_GTK3Version) = Equo_Getversion("x11-libs/gtk+-3")
  EndIf
EndIf

Select SystemInfo(#SystemInfoField_PackageManager)
    Case "yum"
      Output$ = GetOutput("yum", "info gtk2", @Result)
      FieldValue$ = GrabVersionNumber(Output$)
      Position1 = FindString(FieldValue$, " ~ ")
      If (Position1 <> 0)
        FieldValue$ = Left(Output$, Position1)
      EndIf
      Systeminfo(#SystemInfoField_GTK2Version) = FieldValue$
      
      Output$ = GetOutput("yum", "info gtk3", @Result)
      FieldValue$ = GrabVersionNumber(Output$)
      Position1 = FindString(FieldValue$, " ~ ")
      If (Position1 <> 0)
        FieldValue$ = Left(Output$, Position1)
      EndIf
      Systeminfo(#SystemInfoField_GTK3Version) = FieldValue$
      
    Case "apt-get"
      GetGTKVersion_AptGet()
      
    Case "urpmi"
      GetGTKVersion_URPMI()
      
      
EndSelect

GetQTVersion()

;-- get commandline parameters

Parameters = CountProgramParameters()

If (Parameters = 0)
  MyName$ = GetFilePart(ProgramFilename())
  PrintN("Try " + Chr(34) + MyName$ + " tell all" + Chr(34))
  End 996
EndIf

If (Parameters = 2)
  Param1$ = ProgramParameter(0)     ; we treat a leading dash as optional
  If (Left(Param1$, 1) = "-")
    Param1$ = Mid(Param1$, 2)
  EndIf
  
  If (Param1$ = "tell")
    Select ProgramParameter(1)
      Case "all" :
        SystemInfo_Print("Name = " + SystemInfo(#SystemInfoField_Name))
        SystemInfo_Print("Version = " + SystemInfo(#SystemInfoField_Version))
        SystemInfo_Print("ID = " + SystemInfo(#SystemInfoField_ID))
        SystemInfo_Print("ID Like = " + SystemInfo(#SystemInfoField_IDLike))
        SystemInfo_Print("PrettyName = " + SystemInfo(#SystemInfoField_PrettyName))
        SystemInfo_Print("URL = " + SystemInfo(#SystemInfoField_URL))
        SystemInfo_Print("Code Name = " + SystemInfo(#SystemInfoField_CodeName))
        If (SystemInfo(#SystemInfoField_UbuntuCodeName) <> "")
          SystemInfo_Print("Ubuntu Name = " + SystemInfo(#SystemInfoField_UbuntuCodeName))
        EndIf
        If (SystemInfo(#SystemInfoField_GTK2Version) <> "")
          SystemInfo_Print("GTK2Version = " + SystemInfo(#SystemInfoField_GTK2Version))
        EndIf
        If (SystemInfo(#SystemInfoField_GTK3Version) <> "")
          SystemInfo_Print("GTK3Version = " + SystemInfo(#SystemInfoField_GTK3Version))
        EndIf
        If (SystemInfo(#SystemInfoField_QTVERSION) <> "")
          SystemInfo_Print("QTVersion = " + SystemInfo(#SystemInfoField_QTVERSION))
        EndIf
        SystemInfo_Print("VersionID = " + SystemInfo(#SystemInfoField_VersionID))
        SystemInfo_Print("Package Manager = " + SystemInfo(#SystemInfoField_PackageManager))
        If (IPV4$ <> "")
          SystemInfo_Print("IPV4 = " + IPV4$)
        EndIf
        
        If (IPV6$ <> "")
          SystemInfo_Print("IPV6 = " + IPV6$)
        EndIf
        
      Case "ip" :
        If (IPV4$ <> "")
          SystemInfo_Print("IPV4 = " + IPV4$)
        EndIf
        
        If (IPV6$ <> "")
          SystemInfo_Print("IPV6 = " + IPV6$)
        EndIf
        
      Case "gtk2version"
        SystemInfo_Print(SystemInfo(#SystemInfoField_GTK2Version))
        
      Case "gtk3version"
        SystemInfo_Print(SystemInfo(#SystemInfoField_GTK3Version))
        
      Case "name" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_Name))
        
      Case "version" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_Version))
        
      Case "id" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_ID))
        
      Case "idlike" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_IDLike))
        
      Case "prettyname" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_PrettyName))
        
      Case "url" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_URL))
        
      Case "codename" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_CodeName))
        
      Case "ubuntucodename" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_UbuntuCodeName))
        
      Case "versionid" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_VersionID))
        
      Case "gtk2version" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_GTK2Version))
        
      Case "gtk3version" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_GTK3Version))
        
      Case "qtversion" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_QTVERSION))
        
      Case "packagemanager" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_PackageManager))
        
      Case "platformid"
        
      Case "variantid" :
        SystemInfo_Print(SystemInfo(#SystemInfoField_VariantID))
        
      Default :
        SystemInfo_Print("unknown parameter '" + ProgramParameter(1) + "'")
    EndSelect
  Else
    Message$ = "unknown parameter '" + Param1$ + "'. Try 'tell' instead (like 'tell all'."
    SystemInfo_Print(Message$)
  EndIf
EndIf

; Delay(5000)
; IDE Options = PureBasic 5.70 LTS (Linux - x64)
; CursorPosition = 108
; FirstLine = 96
; Folding = ---
; EnableXP